import { SocketEvent } from '@/application/enums'
import Logger, { LoggerType } from '@/application/logger'
import { World } from '@/entities/world'
import SocketManager from '@/managers/socketManager'
import WorldRepository from '@/repositories/worldRepository'
import DateService from '@/services/dateService'

interface GameTimeConfig {
  readonly GAME_SPEED: number
  readonly UPDATE_INTERVAL: number
}

class DateManager {
  private static readonly CONFIG: GameTimeConfig = {
    GAME_SPEED: 8, // 24 game hours / 3 real hours
    UPDATE_INTERVAL: 1000 * 6 // 1 minute
  }

  private readonly logger = Logger.type(LoggerType.APP)
  private readonly worldRepository = new WorldRepository()
  private currentDate = new Date()
  private intervalId: NodeJS.Timeout | null = null

  public async boot(): Promise<void> {
    try {
      this.currentDate = (await this.worldRepository.getFirst())?.date ?? new Date()
      this.startDateLoop()
      this.logger.info('Date manager loaded')
    } catch (error) {
      this.logger.error(`Failed to boot date manager: ${error instanceof Error ? error.message : String(error)}`)
    }
  }

  public async setTime(timeString: string): Promise<void> {
    try {
      const newDate = DateService.parseTimeString(timeString)
      if (!newDate) throw new Error('Invalid time format')

      this.currentDate = newDate
      await this.synchronize()
    } catch (error) {
      this.logger.error(`Failed to set time: ${error instanceof Error ? error.message : String(error)}`)
      throw error
    }
  }

  private startDateLoop(): void {
    this.intervalId = setInterval(async () => {
      const { GAME_SPEED, UPDATE_INTERVAL } = DateManager.CONFIG
      this.currentDate = new Date(this.currentDate.getTime() + DateService.calculateGameTimeAdvance(GAME_SPEED, UPDATE_INTERVAL))
      await this.synchronize()
    }, DateManager.CONFIG.UPDATE_INTERVAL)
  }

  private async synchronize(): Promise<void> {
    try {
      SocketManager.getIO().emit(SocketEvent.DATE, this.currentDate)
      const world = (await this.worldRepository.getFirst()) ?? new World()
      await world.setDate(this.currentDate).save()
    } catch (error) {
      this.logger.error(`Failed to sync date state: ${error instanceof Error ? error.message : String(error)}`)
    }
  }
}

export default new DateManager()
